@@ -1,5 +1,4 @@
 use crate::util::examples::Examples;
-use proc_macro_error::abort;
 use quote::ToTokens;
 use syn::{
     parse::{Parse, ParseStream},
@@ -85,7 +84,7 @@ impl Parse for Response {
         }
 
         if status_code.is_none() {
-            abort!(sp, "status code must be known");
+            return Err(syn::Error::new(sp, "status code must be known"));
         }
 
         Ok(Response {
@@ -149,11 +148,13 @@ impl Parse for DefaultResponse {
         } else if params.len() == 1 {
             ty = Some(params.into_iter().next().unwrap().parse()?);
         } else {
-            abort!(
+            return Err(syn::Error::new(
                 params.span(),
-                r#"expected exactly 1 unnamed parameter, but got {}"#,
-                params.len()
-            );
+                format!(
+                    r#"expected exactly 1 unnamed parameter, but got {}"#,
+                    params.len(),
+                ),
+            ));
         }
 
         Ok(DefaultResponse(Response {
